<?php

/**
 * @file
 * Admin pages and forms for Facebook Application Framework module.
 *
 * @todo
 * Convert fbapp_base_domains and fbapp_preload_fql_multiquery to arrays;
 * Review data subscriptions in face of recent FB API changes;
 * Missing application settings:
 *   http://bugs.developers.facebook.net/show_bug.cgi?id=15911
 *   http://bugs.developers.facebook.net/show_bug.cgi?id=16736
 * Get data subscriptions status from Facebook;
 * Add option to use beta version of Facebook API;
 * Support multiple applications?;
 *
 */

/**
 * Facebook application settings form.
 */
function fbapp_app_settings($form, &$form_state) {
  $fbapp_id = variable_get('fbapp_app_id', '');
  $fbapp_access_token = variable_get('fbapp_app_access_token', '');

  // App Credentials
  // ===============
  $form['fbapp_app_credentials'] = array(
    '#type' => 'fieldset',
    '#title' => t('Application credentials'),
    '#weight' => -5,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_app_credentials']['fbapp_app_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Application ID'),
    '#default_value' => variable_get('fbapp_app_id', ''),
    '#size' => 24,
    '#maxlength' => 15,
    '#description' => t('Your application ID.'),
    '#required' => TRUE,
  );
  $form['fbapp_app_credentials']['fbapp_app_secret'] = array(
    '#type' => 'textfield',
    '#title' => t('Application secret'),
    '#default_value' => variable_get('fbapp_app_secret', ''),
    '#size' => 40,
    '#maxlength' => 32,
    '#description' => t('Your application secret.'),
    '#required' => TRUE,
  );

  // Only display the full form if the application is authenticated.
  if ($fbapp_id != '' && $fbapp_access_token != '') {
    $form['fbapp_app_credentials']['fbapp_app_access_token'] = array(
      '#type' => 'textfield',
      '#title' => t('Access token'),
      '#default_value' => variable_get('fbapp_app_access_token', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The application\'s current access token in use.'),
      '#disabled' => TRUE,
    );

    // About
    // =====
    $form['fbapp_app_about'] = array(
      '#type' => 'fieldset',
      '#title' => t('About'),
      '#weight' => -4,
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    // Basic information
    $form['fbapp_app_about']['fbapp_creator_uid'] = array(
      '#type' => 'textfield',
      '#title' => t('Creator UID'),
      '#value' => variable_get('fbapp_creator_uid', ''),
      '#size' => 24,
      '#maxlength' => 15,
      '#description' => t('The Facebook user ID of the developer who created the application. (This is a read-only property that cannot be set.)'),
      '#disabled' => TRUE,
    );
    $form['fbapp_app_about']['fbapp_application_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Application name'),
      '#default_value' => variable_get('fbapp_application_name', ''),
      '#size' => 40,
      '#maxlength' => 128,
      '#description' => t('The name of your application.'),
    );
    $form['fbapp_app_about']['fbapp_description'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#default_value' => variable_get('fbapp_description', ''),
      '#maxlength' => 512,
      '#description' => t('The description of your application.'),
    );
    $form['fbapp_app_about']['fbapp_icon_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Application icon'),
      '#default_value' => variable_get('fbapp_icon_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to your application\'s icon.'),
    );
    $form['fbapp_app_about']['fbapp_logo_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Application logo'),
      '#default_value' => variable_get('fbapp_logo_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('A URL for your application\'s logo, as shown in product directory and search listings.'),
    );
    $form['fbapp_app_about']['fbapp_connect_logo_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Connect logo'),
      '#default_value' => variable_get('fbapp_connect_logo_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to your site\'s logo; which appears in the Facebook login dialog and on Facebook.com.'),
    );
    $form['fbapp_app_about']['fbapp_locale'] = array(
      '#type' => 'select',
      '#title' => t('Application locale'),
      '#options' => fbapp_locales_list(),
      '#default_value' => variable_get('fbapp_locale', 'en_US'),
      '#description' => t('The default locale for your application.'),
    );
    $form['fbapp_app_about']['fbapp_contact_email'] = array(
      '#type' => 'textfield',
      '#title' => t('Contact email'),
      '#default_value' => variable_get('fbapp_contact_email', ''),
      '#size' => 50,
      '#maxlength' => 128,
      '#description' => t('The email address through which your users can contact you. (Default value is your Facebook email address.)'),
    );
    $form['fbapp_app_about']['fbapp_support_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Support URL'),
      '#default_value' => variable_get('fbapp_support_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to your application\'s support page (for those applications that specify a help page instead of an email address).'),
    );
    $form['fbapp_app_about']['fbapp_about_url'] = array(
      '#type' => 'textfield',
      '#title' => t('About URL'),
      '#default_value' => variable_get('fbapp_about_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('This is the URL to your application\'s About page. About pages are Facebook Pages.'),
    );
    $form['fbapp_app_about']['fbapp_help_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Help URL'),
      '#default_value' => variable_get('fbapp_help_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to your application\'s help page.'),
    );
    $form['fbapp_app_about']['fbapp_privacy_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Privacy URL'),
      '#default_value' => variable_get('fbapp_privacy_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to your application\'s privacy terms.'),
    );
    $form['fbapp_app_about']['fbapp_tos_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Terms of Service URL'),
      '#default_value' => variable_get('fbapp_tos_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to your application\'s Terms of Service.'),
    );

    // Web Site
    // ========
    $form['fbapp_app_website'] = array(
      '#type' => 'fieldset',
      '#title' => t('Web Site'),
      '#weight' => -3,
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    // Core Settings
    $form['fbapp_app_website']['fbapp_connect_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Connect URL'),
      '#default_value' => variable_get('fbapp_connect_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The main URL for a Facebook Connect application.'),
    );
    $form['fbapp_app_website']['fbapp_base_domain'] = array(
      '#type' => 'textfield',
      '#title' => t('Base domain'),
      '#default_value' => variable_get('fbapp_base_domain', ''),
      '#size' => 30,
      '#maxlength' => 128,
      '#description' => t('The base domain for a Facebook-enabled website.'),
    );
    $form['fbapp_app_website']['fbapp_base_domains'] = array(// TODO: This has to be turned to an array.
      '#type' => 'textarea',
      '#title' => t('Base domains'),
      '#default_value' => variable_get('fbapp_base_domains', ''),
      '#maxlength' => 512,
      '#description' => t('An array of base domains for a Facebook-enabled website.'),
    );
    $form['fbapp_app_website']['fbapp_email_domain'] = array(
      '#type' => 'textfield',
      '#title' => t('Email domain'),
      '#default_value' => variable_get('fbapp_email_domain', ''),
      '#size' => 30,
      '#maxlength' => 128,
      '#description' => t('The domain from where you send email to your users.'),
    );

    // Facebook Integration
    // ====================
    $form['fbapp_app_integration'] = array(
      '#type' => 'fieldset',
      '#title' => t('Facebook Integration'),
      '#weight' => -2,
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    // Core Settings
    $form['fbapp_app_integration']['fbapp_canvas_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Canvas name'),
      '#default_value' => variable_get('fbapp_canvas_name', ''),
      '#size' => 20,
      '#maxlength' => 128,
      '#description' => t('The canvas name of your application.'),
      '#field_prefix' => 'https://apps.facebook.com/',
      '#field_suffix' => '/',
    );
    $form['fbapp_app_integration']['fbapp_callback_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Callback URL'),
      '#default_value' => variable_get('fbapp_callback_url', ''),
      '#size' => 60,
      '#maxlength' => 100,
      '#description' => t('Your application\'s callback URL. The callback URL cannot be longer than 100 characters. Once set, this setting cannot be unset.'),
    );
    $form['fbapp_app_integration']['fbapp_secure_callback_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Secure callback URL'),
      '#default_value' => variable_get('fbapp_secure_callback_url', ''),
      '#size' => 60,
      '#maxlength' => 100,
      '#description' => t('The secure_callback_url must be a fully qualified URL beginning with https:// and must point to a directory (i.e., end with a \'/\') or a dynamic page (i.e., have a \'?\' somewhere).'),
    );
    $form['fbapp_app_integration']['fbapp_dashboard_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Dashboard URL'),
      '#default_value' => variable_get('fbapp_dashboard_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('This is the URL to your application from its bookmark on the Applications menu.'),
    );
    $form['fbapp_app_integration']['fbapp_authorize_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Authorization pingback URL'),
      '#default_value' => variable_get('fbapp_authorize_url', ''),
      '#size' => 60,
      '#maxlength' => 100,
      '#description' => t('The URL Facebook pings after a user authorizes your application. The URL cannot be longer than 100 characters.'),
    );
    $form['fbapp_app_integration']['fbapp_post_authorize_redirect_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Authorization redirect URL'),
      '#default_value' => variable_get('fbapp_post_authorize_redirect_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL where a user gets redirected after authorizing your application. If not set, the user will continue on to the page that they were originally going to.'),
    );
    $form['fbapp_app_integration']['fbapp_uninstall_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Uninstall pingback URL'),
      '#default_value' => variable_get('fbapp_uninstall_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL that Facebook pings after a user removes your application.'),
    );
    $form['fbapp_app_integration']['fbapp_info_changed_url'] = array(
      '#type' => 'textfield',
      '#title' => t('App info change pingback URL'),
      '#default_value' => variable_get('fbapp_info_changed_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to ping if a user changes the content of your application info section. Your application can get new content for the section, and set it with profile.setInfo.'),
    );
    $form['fbapp_app_integration']['fbapp_use_iframe'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use IFrame'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_use_iframe', '1'),
      '#description' => t('Indicates whether you render your application with FBML (0) or in an IFrame (1). (Default value is 1.)'),
    );
    $form['fbapp_app_integration']['fbapp_iframe_enable_util'] = array(
      '#type' => 'checkbox',
      '#title' => t('IFrame sizing method'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_iframe_enable_util', '0'),
      '#description' => t('For iframe applications, indicates whether the IFrame is resizable (1) or smartsized (0). (Default value is 0.)'),
    );
    $form['fbapp_app_integration']['fbapp_desktop'] = array(
      '#type' => 'checkbox',
      '#title' => t('Is desktop application'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_desktop', '0'),
      '#description' => t('Indicates whether your application is a Facebook canvas page application (0) or if gets installed on a user\'s desktop (1). (Default value is 0.)'),
    );
    $form['fbapp_app_integration']['fbapp_default_column'] = array(
      '#type' => 'checkbox',
      '#title' => t('Default column'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_default_column', '0'),
      '#description' => t('Indicates whether your application appears in the wide (1) or narrow (0) column of a user\'s Boxes tab. (Default value is 0.)'),
    );
    $form['fbapp_app_integration']['fbapp_private_install'] = array(
      '#type' => 'checkbox',
      '#title' => t('Private install'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_private_install', '0'),
      '#description' => t('Indicates whether you want to disable (1) or enable (0) News Feed and Wall stories when a user installs your application. (Default value is 0.)'),
    );
    $form['fbapp_app_integration']['fbapp_installable'] = array(
      '#type' => 'checkbox',
      '#title' => t('Installable application'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_installable', '1'),
      '#description' => t('Indicates whether a user can (1) or cannot (0) install your application. (Default value is 1.)'),
    );
    $form['fbapp_app_integration']['fbapp_is_mobile'] = array(
      '#type' => 'checkbox',
      '#title' => t('Is mobile'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_is_mobile', '1'),
      '#description' => t('Indicates whether your application can run on a mobile device (1) or not (0). (Default value is 0.)'),
    );
    // Content Options
    $form['fbapp_app_integration']['fbapp_connect_preview_template'] = array(
      '#type' => 'textfield',
      '#title' => t('Request source data URL'),
      '#default_value' => variable_get('fbapp_connect_preview_template', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('For Facebook-enabled websites, the URL from which Facebook pulls the content to display to the user receiving a Facebook request.'),
    );
    $form['fbapp_app_integration']['fbapp_connect_reclaim_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Account recovery URL'),
      '#default_value' => variable_get('fbapp_connect_reclaim_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('For Facebook-enabled websites, the URL you can provide to reclaim an account from a user who deactivated his or her Facebook account.'),
    );
    $form['fbapp_app_integration']['fbapp_publish_self_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Publisher source URL (own)'),
      '#default_value' => variable_get('fbapp_publish_self_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL from which to fetch your application content for use in the Publisher when the user is publishing on his or her own profile.'),
    );
    $form['fbapp_app_integration']['fbapp_publish_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Publisher source URL (others)'),
      '#default_value' => variable_get('fbapp_publish_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL from which to fetch your application content for use in the Publisher when the user is publishing on a friend\'s profile.'),
    );
    $form['fbapp_app_integration']['fbapp_message_action'] = array(
      '#type' => 'textfield',
      '#title' => t('Attachment action label'),
      '#default_value' => variable_get('fbapp_message_action', ''),
      '#size' => 30,
      '#maxlength' => 20,
      '#description' => t('For applications that can create attachments, this is the label for the action that creates the attachment. It cannot be more than 20 characters.'),
    );
    $form['fbapp_app_integration']['fbapp_publish_action'] = array(
      '#type' => 'textfield',
      '#title' => t('Friend\'s Publisher content link'),
      '#default_value' => variable_get('fbapp_publish_action', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The label for the link on a friend\'s profile to an application\'s Publisher content.'),
    );
    $form['fbapp_app_integration']['fbapp_publish_self_action'] = array(
      '#type' => 'textfield',
      '#title' => t('Own Publisher content link'),
      '#default_value' => variable_get('fbapp_publish_self_action', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The label for the link on user\'s own profile to an application\'s Publisher content.'),
    );
    // Page Tabs
    $form['fbapp_app_integration']['fbapp_tab_default_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Default tab name'),
      '#default_value' => variable_get('fbapp_tab_default_name', ''),
      '#size' => 40,
      '#maxlength' => 128,
      '#description' => t('The default label for an application tab when a user first adds it to his or her profile.'),
    );
    $form['fbapp_app_integration']['fbapp_profile_tab_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Tab content URL'),
      '#default_value' => variable_get('fbapp_profile_tab_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL from where Facebook fetches the content for an application tab. The URL should be relative to your canvas page URL at Facebook.'),
    );
    $form['fbapp_app_integration']['fbapp_edit_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Edit link URL'),
      '#default_value' => variable_get('fbapp_edit_url', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('The URL to the **Edit** link displayed on your application\'s profile box. An admin of a Facebook Page that has added your application can follow the URL to edit the application settings for the Page.'),
    );

    // Advanced
    // ========
    $form['fbapp_app_advanced'] = array(
      '#type' => 'fieldset',
      '#title' => t('Advanced'),
      '#weight' => -1,
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    // Authentication
    $form['fbapp_app_advanced']['fbapp_dev_mode'] = array(
      '#type' => 'checkbox',
      '#title' => t('Sandbox mode'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_dev_mode', '1'),
      '#description' => t('Indicates whether sandbox mode is enabled (1) or disabled (0). Only developers can install applications in sandbox mode. (Default value is 1.)'),
    );
    //Security
    $form['fbapp_app_advanced']['fbapp_ip_list'] = array(
      '#type' => 'textarea',
      '#title' => t('Servers IPs'),
      '#default_value' => variable_get('fbapp_ip_list', ''),
      '#maxlength' => 512,
      '#description' => t('For Web-based applications, these are the IP addresses of your servers that can access Facebook\'s servers and serve information to your application.'),
    );
    $form['fbapp_app_advanced']['fbapp_ignore_ip_whitelist_for_ss'] = array(
      '#type' => 'checkbox',
      '#title' => t('Bypass whitelist'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_ignore_ip_whitelist_for_ss', '1'),
      '#description' => t('Indicates whether an iframe application or Facebook Connect site can (1) or cannot (0) bypass a server whitelist. (Default value is 1.)'),
    );
    $form['fbapp_app_advanced']['fbapp_email'] = array(
      '#type' => 'textfield',
      '#title' => t('Administrative email'),
      '#default_value' => variable_get('fbapp_email', ''),
      '#size' => 50,
      '#maxlength' => 128,
      '#description' => t('The email address associated with the application; the email address Facebook uses to contact you about your application. (Default value is your Facebook email address.)'),
    );
    // Advanced Canvas Features
    $form['fbapp_app_advanced']['fbapp_preload_fql'] = array(
      '#type' => 'textfield',
      '#title' => t('Preloaded FQL query'),
      '#default_value' => variable_get('fbapp_preload_fql', ''),
      '#size' => 60,
      '#maxlength' => 128,
      '#description' => t('A preloaded FQL query.'),
    );
    $form['fbapp_app_advanced']['fbapp_preload_fql_multiquery'] = array(// TODO: Make this an array.
      '#type' => 'textarea',
      '#title' => t('Preloaded FQL multiquery'),
      '#default_value' => variable_get('fbapp_preload_fql_multiquery', ''),
      '#maxlength' => 1024,
      '#description' => t('A preloaded FQL multiquery.'),
    );

    // Useless
    // =======
    $form['fbapp_app_useless'] = array(
      '#type' => 'fieldset',
      '#title' => t('Useless'),
      '#weight' => 0,
      '#description' => t('These read only options were left here so people can check its data.'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    $form['fbapp_app_useless']['fbapp_targeted'] = array(
      '#type' => 'checkbox',
      '#title' => t('Demographic Restrictions'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_targeted', '0'),
      '#description' => t('If set to true, indicates that Demographic Restrictions have been set for your application; if false, then the restrictions were never set or were removed. (This is a read-only property that can only be set via the Developer application.) (Default value is 0.)'),
      '#disabled' => TRUE,
    );
    $form['fbapp_app_useless']['fbapp_video_rentals'] = array(
      '#type' => 'checkbox',
      '#title' => t('Video rentals'),
      '#return_value' => 1,
      '#default_value' => variable_get('fbapp_video_rentals', '0'),
      '#description' => t('Indicates whether your application or site publishes Feed stories about video sales (1) or not (0) -- this is for Video Privacy Protection Act conformance. (This is a read-only property that can only be set via the Developer application.)'),
      '#disabled' => TRUE,
    );

    // Load settings from Facebook button.
    $form['load_settings'] = array(
      '#name' => 'fbapp_app_load_settings_button',
      '#type' => 'button',
      '#executes_submit_callback' => TRUE,
      '#submit' => array('fbapp_app_load_settings'),
      '#value' => t('Read from Facebook'),
      '#weight' => 2,
    );
  }

  $form['submit'] = array(
    '#name' => 'submit',
    '#type' => 'button',
    '#executes_submit_callback' => TRUE,
    '#submit' => array('fbapp_app_settings_submit'),
    '#value' => t('Save settings'),
    '#weight' => 1,
  );

  return $form;
}

/**
 * Implements hook_submit().
 */
function fbapp_app_settings_submit($form, &$form_state) {
  // Don't do anything unless required values are set.
  if (isset($form_state['values']['fbapp_app_id']) && isset($form_state['values']['fbapp_app_secret'])) {

    // Save user provided application credentials.
    variable_set('fbapp_app_id', $form_state['values']['fbapp_app_id']);
    variable_set('fbapp_app_secret', $form_state['values']['fbapp_app_secret']);

    // Don't try to save settings without a valid token.
    if (variable_get('fbapp_app_access_token', '') == '') {
      // Get a fresh token.
      $tokendata = fbapp_app_authenticate(variable_get('fbapp_app_id', ''), variable_get('fbapp_app_secret', ''));
      if ($tokendata) {
        variable_set('fbapp_app_access_token', $tokendata['access_token']);
      }
      else {
        drupal_set_message(t('Unable to retreive access token. Check the credentials and try again.'), 'error');
      }
      // Since we didn't even had a valid token, let's fetch the settings from FB.
      fbapp_app_load_settings();
    }
    else {
      $prop_list = fbapp_app_properties_list();
      $properties = array();
      foreach ($prop_list as $item) {
        $prop_data = isset($form_state['values']['fbapp_' . $item]) ? $form_state['values']['fbapp_' . $item] : '';
        // Empty values are allowed, so the user can unset values.
        $properties[$item] = $prop_data;
      }

      // Clear readonly values to prevent them from being submitted.
      unset($properties['creator_uid'], $properties['targeted'], $properties['video_rentals']);
      // If callback_url is submitted with an empty value, FB API returns "500 - Internal server error".
      if ($properties['callback_url'] == '') {
        unset($properties['callback_url']);
      }

      // Save application settings to Facebook.
      $result = fbapp_app_properties_set($properties);
      if ($result == TRUE) {
        drupal_set_message(t('Your application\'s settings were successfully set on Facebook. Remember it may take some time for the settings to propagate through Facebook\'s servers.'));
        // Save (all) settings locally.
        $prop_list = fbapp_app_properties_list();
        $properties = array();
        foreach ($prop_list as $item) {
          variable_set('fbapp_' . $item, $form_state['values']['fbapp_' . $item]);
        }
      }
      else {
        drupal_set_message(t('Application settings were NOT saved.'), 'warning');
      }
    }
  }
}

/**
 * Load application settings from Facebook. Also used as callback by the "Load settings" button.
 */
function fbapp_app_load_settings() {
  $result = fbapp_app_properties_get();
  if ($result == TRUE) {
    drupal_set_message(t('Application settings were retreived from Facebook and applied locally.'), 'status');
  }
  else {
    drupal_set_message(t('Could NOT retreive applcation settings from Facebook.'), 'warning');
  }
}

/**
 * Local application settings form.
 */
function fbapp_local_settings($form, &$form_state) {
  $form['fbapp_cache_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Cache settings'),
    '#weight' => -2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $periods = array(
    0 => 'Don\'t cache objects',
    3600 => '1 hour',
  );
  for ($i = 2; $i < 97; $i++) {
    $periods[3600 * $i] = $i . ' hours';
  }
  $form['fbapp_cache_settings']['fbapp_cache_expiry_time'] = array(
    '#type' => 'select',
    '#title' => t('Object expiry time'),
    '#options' => $periods,
    '#default_value' => variable_get('fbapp_cache_expiry_time', 86400),
    '#description' => t('For how long objects should be cached.'),
    '#required' => FALSE,
  );

  return system_settings_form($form);
}

/**
 * Data subscriptions configuration form.
 */
function fbapp_app_subscriptions($form, &$form_state) {
  $permissions = variable_get('fbapp_permissions', array());

  $form['fbapp_subscriptions_user'] = array(
    '#type' => 'fieldset',
    '#title' => t('User subscriptions'),
    '#weight' => -3,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_subscriptions_user']['fbapp_subscriptions_user_active'] = array(
    '#type' => 'checkbox',
    '#title' => t('User subscriptions active'),
    '#default_value' => variable_get('fbapp_subscriptions_user_active', 0),
    '#description' => t('Enable user data subscription. You need to enable this option and add the "picture" field if you want user\'s pictures to be updated whenever they are changed.'),
  );
  $form['fbapp_subscriptions_user']['fbapp_subscriptions_user_fields'] = array(
    '#type' => 'textarea',
    '#title' => t('User fields'),
    '#default_value' => variable_get('fbapp_subscriptions_user_fields', ''),
    '#maxlength' => 512,
    '#description' => t('A comma separated list of user fields (or connections) to subscribe to. See all user fields in the <a href="@docs">Facebook user docs</a>. You can subscribe to all of the user object properties except verified. Some fields may need specific permissions.', array('@docs' => 'https://developers.facebook.com/docs/reference/api/user/')),
    '#required' => FALSE,
  );
  $form['fbapp_subscriptions_user']['fbapp_subscriptions_user_verified'] = array(
    '#value' => 'This subscription is ' . (variable_get('fbapp_subscriptions_user_verified', 0) == 1 ? 'active.' : 'inactive.'),
  );

  $form['fbapp_subscriptions_permissions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Permissions subscriptions'),
    '#weight' => -2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_subscriptions_permissions']['fbapp_subscriptions_permissions_active'] = array(
    '#type' => 'checkbox',
    '#title' => t('Permissions subscriptions active'),
    '#default_value' => variable_get('fbapp_subscriptions_permissions_active', 0),
    '#description' => t('Enable permissions data subscription.'),
  );
  $form['fbapp_subscriptions_permissions']['fbapp_subscriptions_permissions_fields'] = array(
    '#type' => 'textarea',
    '#title' => t('Permissions to subscribe'),
    '#default_value' => variable_get('fbapp_subscriptions_permissions_fields', ''),
    '#maxlength' => 512,
    '#description' => t('A comma separated list of permissions to subscribe to. See available permissions in the <a href="@docs">Facebook permissions docs</a>. The current selected permissions are: <em>@perms</em>', array('@docs' => 'https://developers.facebook.com/docs/authentication/permissions/', '@perms' => implode(',', array_values($permissions)))),
    '#required' => FALSE,
  );
  $form['fbapp_subscriptions_permissions']['fbapp_subscriptions_permissions_verified'] = array(
    '#value' => 'This subscription is ' . (variable_get('fbapp_subscriptions_permissions_verified', 0) == 1 ? 'active.' : 'inactive.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save subscriptions'),
  );

  return $form;
}

/**
 * Implements hook_submit().
 */
function fbapp_app_subscriptions_submit($form, &$form_state) {
  global $base_url;

  $current_subscriptions_user_status = variable_get('fbapp_subscriptions_user_active', 0);
  $current_subscriptions_permissions_status = variable_get('fbapp_subscriptions_permissions_active', 0);
  $new_subscriptions_user_status = $form_state['values']['fbapp_subscriptions_user_active'];
  $new_subscriptions_permissions_status = $form_state['values']['fbapp_subscriptions_permissions_active'];
  $subscriptions_user_enabled = variable_get('fbapp_subscriptions_user_active', 0) == 0 && $form_state['values']['fbapp_subscriptions_user_active'] == 1;
  $subscriptions_permissions_enabled = variable_get('fbapp_subscriptions_permissions_active', 0) == 0 && $form_state['values']['fbapp_subscriptions_permissions_active'] == 1;
  $subscriptions_user_fields_changed = (variable_get('fbapp_subscriptions_user_fields', '') != $form_state['values']['fbapp_subscriptions_user_fields'] ? TRUE : FALSE);
  $subscriptions_permissions_fields_changed = (variable_get('fbapp_subscriptions_permissions_fields', '') != $form_state['values']['fbapp_subscriptions_permissions_fields'] ? TRUE : FALSE);

  // Avoid HTTPS since Facebook's cURL version apparently doesn't support SNI.
  $callback = str_replace('https', 'http', $base_url . '/fbapp/subscriptions');

  // Check if any of the subscriptions is to be deleted.
  if (($current_subscriptions_user_status && !$new_subscriptions_user_status) || ($current_subscriptions_permissions_status && !$new_subscriptions_permissions_status)) {
    variable_set('fbapp_subscriptions_user_active', 0);
    variable_set('fbapp_subscriptions_permissions_active', 0);
    $params = array(
      'access_token' => variable_get('fbapp_app_access_token', ''),
    );
    $status = fbapp_graph_request(variable_get('fbapp_app_id', '') . '/subscriptions', $params, 'DELETE');
    drupal_set_message(t('Subscriptions deleted.'));
  }
  if ($subscriptions_user_enabled || $subscriptions_user_fields_changed) {
    $subscription_token = 'usertoken_' . REQUEST_TIME;
    variable_set('fbapp_subscriptions_user_active', 1);
    variable_set('fbapp_subscriptions_user_fields', $form_state['values']['fbapp_subscriptions_user_fields']);
    variable_set('fbapp_subscriptions_user_verify_token', $subscription_token);
    variable_set('fbapp_subscriptions_user_verified', 0);
    $params = array(
      'access_token' => variable_get('fbapp_app_access_token', ''),
      'object' => 'user',
      'fields' => $form_state['values']['fbapp_subscriptions_user_fields'],
      'callback_url' => $callback . '?type=user',
      'verify_token' => $subscription_token,
    );
    $status = fbapp_graph_request(variable_get('fbapp_app_id', '') . '/subscriptions', $params, 'POST');
    drupal_set_message(t('User object changes subscribed.'));
  }
  if ($subscriptions_permissions_enabled || $subscriptions_permissions_fields_changed) {
    $subscription_token = 'permstoken_' . REQUEST_TIME;
    variable_set('fbapp_subscriptions_permissions_active', 1);
    variable_set('fbapp_subscriptions_permissions_fields', $form_state['values']['fbapp_subscriptions_permissions_fields']);
    variable_set('fbapp_subscriptions_permissions_verify_token', $subscription_token);
    variable_set('fbapp_subscriptions_permissions_verified', 0);
    $params = array(
      'access_token' => variable_get('fbapp_app_access_token', ''),
      'object' => 'permissions',
      'fields' => $form_state['values']['fbapp_subscriptions_permissions_fields'],
      'callback_url' => $callback . '?type=perms',
      'verify_token' => $subscription_token,
    );
    $status = fbapp_graph_request(variable_get('fbapp_app_id', '') . '/subscriptions', $params, 'POST');
    drupal_set_message(t('Permissions changes subscribed.'));
  }
  $params = array(
    'access_token' => variable_get('fbapp_app_access_token', ''),
  );
}

/**
 * Returns the (known) list of application properties.
 * Reference: http://developers.facebook.com/docs/appproperties/
 */
function fbapp_app_properties_list() {
  return array(
    'creator_uid',
    'application_name',
    'description',
    'icon_url',
    'logo_url',
    'connect_logo_url',
    'locale',
    'contact_email',
    'about_url',
    'help_url',
    'support_url',
    'privacy_url',
    'tos_url',
    'connect_url',
    'base_domain',
    //    'base_domains',
    'email_domain',
    'canvas_name',
    'callback_url',
    'secure_callback_url',
    'dashboard_url',
    'authorize_url',
    'post_authorize_redirect_url',
    'uninstall_url',
    'info_changed_url',
    'use_iframe',
    'iframe_enable_util',
    'desktop',
    'default_column',
    'private_install',
    'installable',
    'is_mobile',
    'connect_preview_template',
    'connect_reclaim_url',
    'publish_self_url',
    'publish_url',
    'message_action',
    'publish_action',
    'publish_self_action',
    'tab_default_name',
    'profile_tab_url',
    'edit_url',
    'dev_mode',
    'ip_list',
    'ignore_ip_whitelist_for_ss',
    'email',
    'preload_fql',
    //    'preload_fql_multiquery',
    'targeted',
    'video_rentals',
  );
}

/**
 * Returns a list of valid Facebook language codes.
 * (From fb.admin.inc)
 */
function fbapp_locales_list() {
  return array(
    'af_ZA' => t('Afrikaans'),
    'sq_AL' => t('Albanian'),
    'ar_AR' => t('Arabic'),
    'hy_AM' => t('Armenian'),
    'ay_BO' => t('Aymara'),
    'az_AZ' => t('Azeri'),
    'eu_ES' => t('Basque'),
    'be_BY' => t('Belarusian'),
    'bn_IN' => t('Bengali'),
    'bs_BA' => t('Bosnian'),
    'bg_BG' => t('Bulgarian'),
    'ca_ES' => t('Catalan'),
    'ck_US' => t('Cherokee'),
    'hr_HR' => t('Croatian'),
    'cs_CZ' => t('Czech'),
    'da_DK' => t('Danish'),
    'nl_NL' => t('Dutch'),
    'nl_BE' => t('Dutch (België)'),
    'en_PI' => t('English (Pirate)'),
    'en_GB' => t('English (UK)'),
    'en_US' => t('English (US)'),
    'en_UD' => t('English (Upside Down)'),
    'eo_EO' => t('Esperanto'),
    'et_EE' => t('Estonian'),
    'fo_FO' => t('Faroese'),
    'tl_PH' => t('Filipino'),
    'fi_FI' => t('Finnish'),
    'fb_FI' => t('Finnish (test)'),
    'fr_FR' => t('French (France)'),
    'fr_CA' => t('French (Canada)'),
    'gl_ES' => t('Galician'),
    'ka_GE' => t('Georgian'),
    'de_DE' => t('German'),
    'el_GR' => t('Greek'),
    'gn_PY' => t('Guaraní'),
    'gu_IN' => t('Gujarati'),
    'he_IL' => t('Hebrew'),
    'hi_IN' => t('Hindi'),
    'hu_HU' => t('Hungarian'),
    'is_IS' => t('Icelandic'),
    'id_ID' => t('Indonesian'),
    'ga_IE' => t('Irish'),
    'it_IT' => t('Italian'),
    'ja_JP' => t('Japanese'),
    'jv_ID' => t('Javanese'),
    'kn_IN' => t('Kannada'),
    'kk_KZ' => t('Kazakh'),
    'km_KH' => t('Khmer'),
    'tl_ST' => t('Klingon'),
    'ko_KR' => t('Korean'),
    'ku_TR' => t('Kurdish'),
    'la_VA' => t('Latin'),
    'lv_LV' => t('Latvian'),
    'fb_LT' => t('Leet Speak'),
    'li_NL' => t('Limburgish'),
    'lt_LT' => t('Lithuanian'),
    'mk_MK' => t('Macedonian'),
    'mg_MG' => t('Malagasy'),
    'ms_MY' => t('Malay'),
    'ml_IN' => t('Malayalam'),
    'mt_MT' => t('Maltese'),
    'mr_IN' => t('Marathi'),
    'mn_MN' => t('Mongolian'),
    'ne_NP' => t('Nepali'),
    'se_NO' => t('Northern Sámi'),
    'nb_NO' => t('Norwegian (bokmal)'),
    'nn_NO' => t('Norwegian (nynorsk)'),
    'ps_AF' => t('Pashto'),
    'fa_IR' => t('Persian'),
    'pl_PL' => t('Polish'),
    'pt_PT' => t('Portuguese (Portugal)'),
    'pt_BR' => t('Portuguese (Brazil)'),
    'pa_IN' => t('Punjabi'),
    'qu_PE' => t('Quechua'),
    'ro_RO' => t('Romanian'),
    'rm_CH' => t('Romansh'),
    'ru_RU' => t('Russian'),
    'sa_IN' => t('Sanskrit'),
    'sr_RS' => t('Serbian'),
    'zh_CN' => t('Simplified Chinese (China)'),
    'sk_SK' => t('Slovak'),
    'sl_SI' => t('Slovenian'),
    'so_SO' => t('Somali'),
    'es_LA' => t('Spanish'),
    'es_CL' => t('Spanish (Chile)'),
    'es_CO' => t('Spanish (Colombia)'),
    'es_MX' => t('Spanish (Mexico)'),
    'es_ES' => t('Spanish (Spain)'),
    'es_VE' => t('Spanish (Venezuela)'),
    'sw_KE' => t('Swahili'),
    'sv_SE' => t('Swedish'),
    'sy_SY' => t('Syriac'),
    'tg_TJ' => t('Tajik'),
    'ta_IN' => t('Tamil'),
    'tt_RU' => t('Tatar'),
    'te_IN' => t('Telugu'),
    'th_TH' => t('Thai'),
    'zh_HK' => t('Traditional Chinese (Hong Kong)'),
    'zh_TW' => t('Traditional Chinese (Taiwan)'),
    'tr_TR' => t('Turkish'),
    'uk_UA' => t('Ukrainian'),
    'ur_PK' => t('Urdu'),
    'uz_UZ' => t('Uzbek'),
    'vi_VN' => t('Vietnamese'),
    'cy_GB' => t('Welsh'),
    'xh_ZA' => t('Xhosa'),
    'yi_DE' => t('Yiddish'),
    'zu_ZA' => t('Zulu'),
  );
}

