<?php

/**
 * @file
 * Rules integration for the Facebook Application Framework module.
 *
 * @todo
 * Implement posting events (including RSVP's) and checkins;
 *
 */

/**
 * Implements hook_rules_event_info().
 */
function fbapp_rules_rules_event_info() {
  return array(
    'fbapp_user_connected' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('User has connected his Facebook account'),
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('User that allowed Facebook App access'),
        ),
      ),
    ),
    'fbapp_user_disconnected' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('User has disconnected his Facebook account.'),
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('User that revoked Facebook App access.'),
        ),
      ),
    ),
    'fbapp_user_login' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('User has logged in via Facebook'),
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('User who has logged in via Facebook.'),
        ),
      ),
    ),
    'fbapp_user_data_modified' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('An user has changed some of his/hers data on Facebook'),
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('The user whose data changed.'),
        ),
        'fields' => array(
          'type' => 'string',
          'label' => t('The fields that were affected.'),
        ),
        'time' => array(
          'type' => 'date',
          'label' => t('The time of the changes.'),
        ),
      ),
    ),
    'fbapp_user_permissions_modified' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('An user has changed granted permissions on Facebook'),
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('The user whose data changed.'),
        ),
        'permissions' => array(
          'type' => 'string',
          'label' => t('The permissions that were affected.'),
        ),
        'time' => array(
          'type' => 'date',
          'label' => t('The time of the changes.'),
        ),
      ),
    ),
  );
}

/**
 * Implements hook_rules_action_info().
 */
function fbapp_rules_rules_action_info() {
  return array(
    'fbapp_rules_publish_post' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Publish post to Facebook'),
      'parameter' => array(
        'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application creating the post.'),
        ), 
        'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the target feed to post on (user\'s wall, page, app, group, etc.)'),
        ), 
        'message' => array(
          'type' => 'text',
          'label' => t('Message'),
          'description' => t('The post text.'),
        ), 
      ),
      'provides' => array(
        'post_id' => array(
          'type' => 'string',
          'label' => t('Created Post ID'),
        ),
      ),
    ),
    'fbapp_rules_publish_event' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Publish event to Facebook'),
      'parameter' => array(
        'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application creating the post.'),
        ), 
        'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the target feed to post on (user\'s wall, page, app, group, etc.)'),
        ), 
        'name' => array(
          'type' => 'text',
          'label' => t('Name'),
          'description' => t('The name of the event.'),
        ), 
        'start_time' => array(
          'type' => 'text',
          'label' => t('Start Time'),
          'description' => t('Event start time, in ISO-8601.'),
        ), 
        'end_time' => array(
          'type' => 'text',
          'label' => t('End Time'),
          'description' => t('Event end time, in ISO-8601.'),
        ), 
        'description' => array(
          'type' => 'text',
          'label' => t('Description'),
          'description' => t('Event description.'),
        ), 
        'location' => array(
          'type' => 'text',
          'label' => t('Event location'),
          'description' => t('Event location.'),
        ), 
        'location_id' => array(
          'type' => 'text',
          'label' => t('Event location ID'),
          'description' => t('Facebook Place ID of the place the Event is taking place.'),
          'optional' => TRUE,
        ), 
      ),
      'provides' => array(
        'event_id' => array(
          'type' => 'string',
          'label' => t('Created Event ID'),
        ),
      ),
    ),
    'fbapp_rules_publish_link' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Publish link to Facebook'),
      'parameter' => array(
        'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application creating the post.'),
        ), 
        'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the target feed to post on (user\'s wall, page, app, group, etc.)'),
        ), 
        'link' => array(
          'type' => 'text',
          'label' => t('Link URL'),
          'description' => t('The Link URL.'),
        ), 
        'message' => array(
          'type' => 'text',
          'label' => t('Message'),
          'description' => t('The post text.'),
        ), 
        'picture' => array(
          'type' => 'text',
          'label' => t('Picture'),
          'description' => t('A URL to the thumbnail image used in the link post.'),
        ), 
        'name' => array(
          'type' => 'text',
          'label' => t('Link name'),
          'description' => t('The name of the link.'),
        ), 
        'caption' => array(
          'type' => 'text',
          'label' => t('Caption'),
          'description' => t('The link caption.'),
        ), 
        'description' => array(
          'type' => 'text',
          'label' => t('Message'),
          'description' => t('A description of the link (appears beneath the link caption).'),
        ), 
      ),
      'provides' => array(
        'link_id' => array(
          'type' => 'string',
          'label' => t('Created Link ID'),
        ),
      ),
    ),
    'fbapp_rules_publish_photo' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Publish photo to Facebook'),
      'parameter' => array(
	      'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application creating the post.'),
	      ), 
	      'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the target feed to post on (user\'s wall, page, app, group, etc.)'),
	      ), 
	      'source' => array(
          'type' => 'text',
          'label' => t('Image URL'),
          'description' => t('Absolute physical path to the image file to post, starting with leading "/".'),
	      ), 
	      'message' => array(
          'type' => 'text',
          'label' => t('Message'),
          'description' => t('The photo caption.'),
	      ), 
	      'no_story' => array(
          'type' => 'boolean',
          'label' => t('Don\'t publish to user\'s wall'),
          'description' => t('Check this option to inhibit publishing a story on the user\'s wall. If checked, the user will be notified that the application has posted a photo on his behalf.'),
	      ), 
      ),
      'provides' => array(
        'photo_id' => array(
          'type' => 'string',
          'label' => t('Created Photo ID'),
        ),
      ),
    ),
    'fbapp_rules_comment_object' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Comment an object on Facebook'),
      'parameter' => array(
	      'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application creating the post.'),
	      ), 
	      'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the target feed to post on (user\'s wall, page, app, group, etc.)'),
	      ), 
	      'message' => array(
          'type' => 'text',
          'label' => t('Message'),
          'description' => t('The photo caption.'),
	      ), 
      ),
      'provides' => array(
        'comment_id' => array(
          'type' => 'string',
          'label' => t('Created Comment ID'),
        ),
      ),
    ),
    'fbapp_rules_like_object' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Like an object on Facebook'),
      'eval input' => array('access_token', 'target'),
      'parameter' => array(
	      'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application executing the like action.'),
	      ), 
	      'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the object to like.'),
	      ), 
      ),
    ),
    'fbapp_rules_unlike_object' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Unlike an object on Facebook'),
      'parameter' => array(
	      'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application deleting the like.'),
	      ), 
	      'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the object to unlike.'),
	      ), 
      ),
    ),
    'fbapp_rules_delete_object' => array(
      'group' => t('Facebook Application Framework'),
      'module' => 'fbapp_rules',
      'label' => t('Delete object from Facebook'),
      'parameter' => array(
	      'access_token' => array(
          'type' => 'text',
          'label' => t('Access token'),
          'description' => t('The access token of the user or application deleting the obejct.'),
	      ), 
	      'target' => array(
          'type' => 'text',
          'label' => t('Target ID'),
          'description' => t('Facebook ID of the object to delete.'),
	      ), 
      ),
    ),
  );
}

/**
 * Publish post to Facebook.
 */
function fbapp_rules_publish_post($access_token, $target, $message) {
  $parameters = array(
    'access_token' => $access_token,
    'message' => $message,
    'actions' => '',
  );
  $post = fbapp_graph_request("{$target}/feed", $parameters, 'POST');
  return array('post_id' => $post['id']);
}

/**
 * Publish event to Facebook.
 */
function fbapp_rules_publish_event($access_token, $target, $name, $start_time, $end_time, $description, $location, $location_id) {
  $parameters = array(
    'access_token' => $access_token,
    'name' => $name,
    'start_time' => $start_time,
    'end_time' => $end_time,
    'description' => $description,
    'location' => $location,
    'location_id' => $location_id,
  );
  $event = fbapp_graph_request("{$target}/events", $parameters, 'POST');
  return array('event_id' => $event['id']);
}

/**
 * Publish link to Facebook.
 */
function fbapp_rules_publish_link($access_token, $target, $link, $message, $picture, $name, $caption, $description) {
  $parameters = array(
    'access_token' => $access_token,
    'message' => $message,
    'link' => $link,
    'picture' => $picture,
    'name' => $name,
    'caption' => $caption,
    'description' => $description,
  );
  // This should be posted to /links to have a "share" link available, but doesn't work.
  $link = fbapp_graph_request("{$target}/feed", $parameters, 'POST');
  return array('link_id' => $link['id']);
}

/**
 * Publish photo to Facebook.
 */
function fbapp_rules_publish_photo($access_token, $target, $source, $message, $no_story) {
  $parameters = array(
    'access_token' => $access_token,
    'source' => '@' . realpath('.' . base_path()) . $source,
    'message' => $message,
    // Used to get the photo ID in return.
    'fields' => 'id',
  );
  if ($no_story == 1 || $no_story == TRUE) {
    $parameters['no_story'] = 1;
  }
  $photo = fbapp_graph_request("{$target}/photos", $parameters, 'POST');
  // Return the created photo ID.
  return array('photo_id' => $photo['id']);
}

/**
 * Comment object on Facebook.
 */
function fbapp_rules_comment_object($access_token, $target, $message) {
  $parameters = array(
    'access_token' => $access_token,
    'message' => $message,
    // Used to get the comment ID in return.
    'fields' => 'id',
  );
  $comment = fbapp_graph_request("{$target}/comments", $parameters, 'POST');
  // Return the created comment ID.
  return array('comment_id' => $comment['id']);
}

/**
 * Like object on Facebook.
 */
function fbapp_rules_like_object($access_token, $target) {
  $parameters = array(
    'access_token' => $access_token,
  );
  fbapp_graph_request("{$target}/likes", $parameters, 'POST');
}

/**
 * Unlike object on Facebook.
 */
function fbapp_rules_unlike_object($access_token, $target) {
  $parameters = array(
    'access_token' => $access_token,
  );
  fbapp_graph_request("{$target}/likes", $parameters, 'DELETE');
}

/**
 * Delete object from Facebook.
 */
function fbapp_rules_delete_object($access_token, $target) {
  if ($target == '') {
    return;
  }
  $parameters = array(
    'access_token' => $access_token,
  );
  fbapp_graph_request("{$target}", $parameters, 'DELETE');
}

