<?php

/**
 * @file
 * Admin pages and forms for Facebook Application Framework User integration module.
 *
 * @todo
 * Create form to associate FB data to Content Profile fields;
 * Set required permissions for selected options;
 * Set permissions required by other modules automatically;
 * Create permission sets per role?;
 *
 */

/**
 * Facebook application permissions required.
 */
function fbapp_user_permissions($form, &$form_state) {
  // Data permissions
  // ================
  $form['fbapp_data_permissions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Data permissions'),
    '#weight' => -3,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_about_me'] = array(
    '#type' => 'checkboxes',
    '#title' => t('About me'),
    '#default_value' => array(
      'user_about_me' => fbapp_user_get_permission('user_about_me') ? 'user_about_me' : 0,
      'friends_about_me' => fbapp_user_get_permission('friends_about_me') ? 'friends_about_me' : 0,
    ),
    '#options' => array(
      'user_about_me' => t('User about me'),
      'friends_about_me' => t('Friends about me'),
    ),
    '#description' => t('Provides access to the "About Me" section of the profile in the about property.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_birthday'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Birthday'),
    '#default_value' => array(
      'user_birthday' => fbapp_user_get_permission('user_birthday') ? 'user_birthday' : 0,
      'friends_birthday' => fbapp_user_get_permission('friends_birthday') ? 'friends_birthday' : 0,
    ),
    '#options' => array(
      'user_birthday' => t('User birthday'),
      'friends_birthday' => t('Friends birthday'),
    ),
    '#description' => t('Provides access to the birthday with year as the birthday_date property.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_education_history'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Education history'),
    '#default_value' => array(
      'user_education_history' => fbapp_user_get_permission('user_education_history') ? 'user_education_history' : 0,
      'friends_education_history' => fbapp_user_get_permission('friends_education_history') ? 'friends_education_history' : 0,
    ),
    '#options' => array(
      'user_education_history' => t('User education history'),
      'friends_education_history' => t('Friends education history'),
    ),
    '#description' => t('Provides access to education history as the education property.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_hometown'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Hometown'),
    '#default_value' => array(
      'user_hometown' => fbapp_user_get_permission('user_hometown') ? 'user_hometown' : 0,
      'friends_hometown' => fbapp_user_get_permission('friends_hometown') ? 'friends_hometown' : 0,
    ),
    '#options' => array(
      'user_hometown' => t('User hometown'),
      'friends_hometown' => t('Friends hometown'),
    ),
    '#description' => t('Provides access to the user\'s hometown in the hometown property.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_location'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Location'),
    '#default_value' => array(
      'user_location' => fbapp_user_get_permission('user_location') ? 'user_location' : 0,
      'friends_location' => fbapp_user_get_permission('friends_location') ? 'friends_location' : 0,
    ),
    '#options' => array(
      'user_location' => t('User location'),
      'friends_location' => t('Friends location'),
    ),
    '#description' => t('Provides access to the user\'s current location as the location property.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_work_history'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Work history'),
    '#default_value' => array(
      'user_work_history' => fbapp_user_get_permission('user_work_history') ? 'user_work_history' : 0,
      'friends_work_history' => fbapp_user_get_permission('friends_work_history') ? 'friends_work_history' : 0,
    ),
    '#options' => array(
      'user_work_history' => t('User work history'),
      'friends_work_history' => t('Friends work history'),
    ),
    '#description' => t('Provides access to work history as the work property.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_email'] = array(
    '#type' => 'checkbox',
    '#title' => t('Email'),
    '#default_value' => fbapp_user_get_permission('email') ? 1 : 0,
    '#description' => t('Provides access to the user\'s primary email address in the email property. Do not spam users. Your use of email must comply both with Facebook policies and with the CAN-SPAM Act.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_website'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Website'),
    '#default_value' => array(
      'user_website' => fbapp_user_get_permission('user_website') ? 'user_website' : 0,
      'friends_website' => fbapp_user_get_permission('friends_website') ? 'friends_website' : 0,
    ),
    '#options' => array(
      'user_website' => t('User website'),
      'friends_website' => t('Friends website'),
    ),
    '#description' => t('Provides access to the user\'s web site URL.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_relationships'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Relationships'),
    '#default_value' => array(
      'user_relationships' => fbapp_user_get_permission('user_relationships') ? 'user_relationships' : 0,
      'friends_relationships' => fbapp_user_get_permission('friends_relationships') ? 'friends_relationships' : 0,
    ),
    '#options' => array(
      'user_relationships' => t('User relationships'),
      'friends_relationships' => t('Friends relationships'),
    ),
    '#description' => t('Provides access to the user\'s family and personal relationships and relationship status.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_relationship_details'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Relationship details'),
    '#default_value' => array(
      'user_relationship_details' => fbapp_user_get_permission('user_relationship_details') ? 'user_relationship_details' : 0,
      'friends_relationship_details' => fbapp_user_get_permission('friends_relationship_details') ? 'friends_relationship_details' : 0,
    ),
    '#options' => array(
      'user_relationship_details' => t('User relationship details'),
      'friends_relationship_details' => t('Friends relationship details'),
    ),
    '#description' => t('Provides access to the user\'s relationship preferences.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_religion_politics'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Religion/politics'),
    '#default_value' => array(
      'user_religion_politics' => fbapp_user_get_permission('user_religion_politics') ? 'user_religion_politics' : 0,
      'friends_religion_politics' => fbapp_user_get_permission('friends_religion_politics') ? 'friends_religion_politics' : 0,
    ),
    '#options' => array(
      'user_religion_politics' => t('User religion/politics'),
      'friends_religion_politics' => t('Friends religion/politics'),
    ),
    '#description' => t('Provides access to the user\'s religious and political affiliations.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_status'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Status'),
    '#default_value' => array(
      'user_status' => fbapp_user_get_permission('user_status') ? 'user_status' : 0,
      'friends_status' => fbapp_user_get_permission('friends_status') ? 'friends_status' : 0,
    ),
    '#options' => array(
      'user_status' => t('User status'),
      'friends_status' => t('Friends status'),
    ),
    '#description' => t('Provides access to the user\'s most recent status message.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_online_presence'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Online presence'),
    '#default_value' => array(
      'user_online_presence' => fbapp_user_get_permission('user_online_presence') ? 'user_online_presence' : 0,
      'friends_online_presence' => fbapp_user_get_permission('friends_online_presence') ? 'friends_online_presence' : 0,
    ),
    '#options' => array(
      'user_online_presence' => t('User online presence'),
      'friends_online_presence' => t('Friends online presence'),
    ),
    '#description' => t('Provides access to the user\'s online/offline presence.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_photos'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Photos'),
    '#default_value' => array(
      'user_photos' => fbapp_user_get_permission('user_photos') ? 'user_photos' : 0,
      'friends_photos' => fbapp_user_get_permission('friends_photos') ? 'friends_photos' : 0,
    ),
    '#options' => array(
      'user_photos' => t('User photos'),
      'friends_photos' => t('Friends photos'),
    ),
    '#description' => t('Provides access to the photos the user has uploaded.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_videos'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Videos'),
    '#default_value' => array(
      'user_videos' => fbapp_user_get_permission('user_videos') ? 'user_videos' : 0,
      'friends_videos' => fbapp_user_get_permission('friends_videos') ? 'friends_videos' : 0,
    ),
    '#options' => array(
      'user_videos' => t('User videos'),
      'friends_videos' => t('Friends videos'),
    ),
    '#description' => t('Provides access to the videos the user has uploaded.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_checkins'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Checkins'),
    '#default_value' => array(
      'user_checkins' => fbapp_user_get_permission('user_checkins') ? 'user_checkins' : 0,
      'friends_checkins' => fbapp_user_get_permission('friends_checkins') ? 'friends_checkins' : 0,
    ),
    '#options' => array(
      'user_checkins' => t('User checkins'),
      'friends_checkins' => t('Friends checkins'),
    ),
    '#description' => t('Provides read access to the authorized user\'s check-ins or a friend\'s check-ins that the user can see.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_activities'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Activities'),
    '#default_value' => array(
      'user_activities' => fbapp_user_get_permission('user_activities') ? 'user_activities' : 0,
      'friends_activities' => fbapp_user_get_permission('friends_activities') ? 'friends_activities' : 0,
    ),
    '#options' => array(
      'user_activities' => t('User activities'),
      'friends_activities' => t('Friends activities'),
    ),
    '#description' => t('Provides access to the user\'s list of activities as the activities connection.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_events'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Events'),
    '#default_value' => array(
      'user_events' => fbapp_user_get_permission('user_events') ? 'user_events' : 0,
      'friends_events' => fbapp_user_get_permission('friends_events') ? 'friends_events' : 0,
    ),
    '#options' => array(
      'user_events' => t('User events'),
      'friends_events' => t('Friends events'),
    ),
    '#description' => t('Provides access to the list of events the user is attending as the events connection.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_groups'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Groups'),
    '#default_value' => array(
      'user_groups' => fbapp_user_get_permission('user_groups') ? 'user_groups' : 0,
      'friends_groups' => fbapp_user_get_permission('friends_groups') ? 'friends_groups' : 0,
    ),
    '#options' => array(
      'user_groups' => t('User groups'),
      'friends_groups' => t('Friends groups'),
    ),
    '#description' => t('Provides access to the list of groups the user is a member of as the groups connection.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_interests'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Interests'),
    '#default_value' => array(
      'user_interests' => fbapp_user_get_permission('user_interests') ? 'user_interests' : 0,
      'friends_interests' => fbapp_user_get_permission('friends_interests') ? 'friends_interests' : 0,
    ),
    '#options' => array(
      'user_interests' => t('User interests'),
      'friends_interests' => t('Friends interests'),
    ),
    '#description' => t('Provides access to the user\'s list of interests as the interests connection.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_likes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Likes'),
    '#default_value' => array(
      'user_likes' => fbapp_user_get_permission('user_likes') ? 'user_likes' : 0,
      'friends_likes' => fbapp_user_get_permission('friends_likes') ? 'friends_likes' : 0,
    ),
    '#options' => array(
      'user_likes' => t('User likes'),
      'friends_likes' => t('Friends likes'),
    ),
    '#description' => t('Provides access to the list of all of the pages the user has liked as the likes connection.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_notes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Notes'),
    '#default_value' => array(
      'user_notes' => fbapp_user_get_permission('user_notes') ? 'user_notes' : 0,
      'friends_notes' => fbapp_user_get_permission('friends_notes') ? 'friends_notes' : 0,
    ),
    '#options' => array(
      'user_notes' => t('User notes'),
      'friends_notes' => t('Friends notes'),
    ),
    '#description' => t('Provides access to the user\'s notes as the notes connection.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_photo_video_tags'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Photo/video tags'),
    '#default_value' => array(
      'user_photo_video_tags' => fbapp_user_get_permission('user_photo_video_tags') ? 'user_photo_video_tags' : 0,
      'friends_photo_video_tags' => fbapp_user_get_permission('friends_photo_video_tags') ? 'friends_photo_video_tags' : 0,
    ),
    '#options' => array(
      'user_photo_video_tags' => t('User photo/video tags'),
      'friends_photo_video_tags' => t('Friends photo/video tags'),
    ),
    '#description' => t('Provides access to the photos the user has been tagged in as the photos connection.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_friendlists'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Friendlists'),
    '#default_value' => array(
      'read_friendlists' => fbapp_user_get_permission('read_friendlists') ? 'read_friendlists' : 0,
      'manage_friendlists' => fbapp_user_get_permission('manage_friendlists') ? 'manage_friendlists' : 0,
    ),
    '#options' => array(
      'read_friendlists' => t('Read friendlists'),
      'manage_friendlists' => t('Manage friendlists'),
    ),
    '#description' => t('Provides access to any friend lists the user created. All user\'s friends are provided as part of basic data, this extended permission grants access to the lists of friends a user has created, and should only be requested if your application utilizes lists of friends.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_read_stream'] = array(
    '#type' => 'checkbox',
    '#title' => t('Read stream'),
    '#default_value' => fbapp_user_get_permission('read_stream') ? 1 : 0,
    '#description' => t('Provides access to all the posts in the user\'s News Feed and enables your application to perform searches against the user\'s News Feed.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_read_insights'] = array(
    '#type' => 'checkbox',
    '#title' => t('Read insights'),
    '#default_value' => fbapp_user_get_permission('read_insights') ? 1 : 0,
    '#description' => t('Provides read access to the Insights data for pages, applications, and domains the user owns.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_read_mailbox'] = array(
    '#type' => 'checkbox',
    '#title' => t('Read mailbox'),
    '#default_value' => fbapp_user_get_permission('read_mailbox') ? 1 : 0,
    '#description' => t('Provides the ability to read from a user\'s Facebook Inbox.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_read_requests'] = array(
    '#type' => 'checkbox',
    '#title' => t('Read requests'),
    '#default_value' => fbapp_user_get_permission('read_requests') ? 1 : 0,
    '#description' => t('Provides read access to the user\'s friend requests.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_xmpp_login'] = array(
    '#type' => 'checkbox',
    '#title' => t('XMPP login'),
    '#default_value' => fbapp_user_get_permission('xmpp_login') ? 1 : 0,
    '#description' => t('Provides applications that integrate with Facebook Chat the ability to log in users.'),
  );
  $form['fbapp_data_permissions']['fbapp_user_perm_ads_management'] = array(
    '#type' => 'checkbox',
    '#title' => t('Ads management'),
    '#default_value' => fbapp_user_get_permission('ads_management') ? 1 : 0,
    '#description' => t('Provides the ability to manage ads and call the Facebook Ads API on behalf of a user.'),
  );

  // Publishing permissions
  // ======================
  $form['fbapp_publishing_permissions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Publishing permissions'),
    '#weight' => -2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_publishing_permissions']['fbapp_user_perm_publish_stream'] = array(
    '#type' => 'checkbox',
    '#title' => t('Publish stream'),
    '#default_value' => fbapp_user_get_permission('publish_stream') ? 1 : 0,
    '#description' => t('Enables your application to post content, comments, and likes to a user\'s stream and to the streams of the user\'s friends. With this permission, you can publish content to a user\'s feed at any time, without requiring offline_access. However, please note that Facebook recommends a user-initiated sharing model.'),
  );
  $form['fbapp_publishing_permissions']['fbapp_user_perm_create_event'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create event'),
    '#default_value' => fbapp_user_get_permission('create_event') ? 1 : 0,
    '#description' => t('Enables your application to create and modify events on the user\'s behalf.'),
  );
  $form['fbapp_publishing_permissions']['fbapp_user_perm_rsvp_event'] = array(
    '#type' => 'checkbox',
    '#title' => t('RSVP event'),
    '#default_value' => fbapp_user_get_permission('rsvp_event') ? 1 : 0,
    '#description' => t('Enables your application to RSVP to events on the user\'s behalf.'),
  );
  $form['fbapp_publishing_permissions']['fbapp_user_perm_sms'] = array(
    '#type' => 'checkbox',
    '#title' => t('SMS'),
    '#default_value' => fbapp_user_get_permission('sms') ? 1 : 0,
    '#description' => t('Enables your application to send messages to the user and respond to messages from the user via text message.'),
  );
  $form['fbapp_publishing_permissions']['fbapp_user_perm_offline_access'] = array(
    '#type' => 'checkbox',
    '#title' => t('Offline access'),
    '#default_value' => fbapp_user_get_permission('offline_access') ? 1 : 0,
    '#description' => t('Enables your application to perform authorized requests on behalf of the user at any time. By default, most access tokens expire after a short time period to ensure applications only make requests on behalf of the user when they are actively using the application. This permission makes the access token returned by our OAuth endpoint long-lived.'),
  );
  $form['fbapp_publishing_permissions']['fbapp_user_perm_publish_checkins'] = array(
    '#type' => 'checkbox',
    '#title' => t('Publish checkins'),
    '#default_value' => fbapp_user_get_permission('publish_checkins') ? 1 : 0,
    '#description' => t('Enables your application to perform checkins on behalf of the user.'),
  );

  // Page permissions
  // ======================
  $form['fbapp_page_permissions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Page permissions'),
    '#weight' => -1,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_page_permissions']['fbapp_user_perm_manage_pages'] = array(
    '#type' => 'checkbox',
    '#title' => t('Manage pages'),
    '#default_value' => fbapp_user_get_permission('manage_pages') ? 1 : 0,
    '#description' => t('Enables your application to retrieve access_tokens for pages the user administrates. The access tokens can be queried using the "accounts" connection in the Graph API. This permission is only compatible with the Graph API.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}

/**
 * Returns the status of the requested permission.
 */
function fbapp_user_get_permission($perm) {
  $permissions = variable_get('fbapp_user_permissions', '');
  return is_array($permissions) && in_array($perm, $permissions) ? TRUE : FALSE;
}

/**
 * Implements hook_submit().
 */
function fbapp_user_permissions_submit($form, &$form_state) {
  // Iterate through posted values, build permissions array and save it.
  foreach ($form_state['values'] as $key => $value) {
    if (strncmp($key, 'fbapp_user_perm', 15) == 0) {
      if (is_array($value)) {
        foreach ($value as $skey => $svalue) {
          if ($svalue === $skey) {
            $permissions[] = $skey;
          }
        }
      }
      else {
        if ($value == 1) {
          $permissions[] = str_replace('fbapp_user_perm_', '', $key);
        }
      }
    }
  }
  variable_set('fbapp_user_permissions', $permissions);
  drupal_set_message(t('Permissions saved.'));
}

/**
 * User settings form.
 */
function fbapp_user_settings($form, &$form_state) {
  $form['fbapp_user_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('User account settings'),
    '#weight' => -4,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_user_settings']['fbapp_user_login_mode'] = array(
    '#type' => 'select',
    '#title' => t('Login mode'),
    '#options' => array(
      '0' => t('Disable Facebook authentication'),
      '1' => t('Use only Facebook authentication'),
      '2' => t('Use local and Facebook authentication'),
    ),
    '#default_value' => variable_get('fbapp_user_login_mode', '0'),
    '#description' => t('The authentication system(s) to use.'),
    '#required' => FALSE,
  );
  $form['fbapp_user_settings']['fbapp_user_registration_mode'] = array(
    '#type' => 'select',
    '#title' => t('Registration mode'),
    '#options' => array(
      '0' => t('User must register locally'),
      '1' => t('Register user automatically'),
    ),
    '#default_value' => variable_get('fbapp_user_registration_mode', '0'),
    '#description' => t('Require user to fill in the registration form, or register the user automatically with data from Facebook. The "email" permission is required to create accounts automatically.'),
    '#required' => FALSE,
  );
  $form['fbapp_user_profile'] = array(
    '#type' => 'fieldset',
    '#title' => t('User profile settings'),
    '#weight' => -3,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['fbapp_user_profile']['fbapp_user_profile_name'] = array(
    '#type' => 'select',
    '#title' => t('Username field'),
    '#options' => array(
      'username' => t('Username'),
      'name' => t('Full name'),
      'email' => t('Email'),
    ),
    '#default_value' => variable_get('fbapp_user_profile_name', 'username'),
    '#description' => t('Select the Facebook user field to use as username in Drupal.'),
    '#required' => FALSE,
  );
  $form['fbapp_user_profile']['fbapp_user_profile_picture'] = array(
    '#type' => 'select',
    '#title' => t('Picture dimensions'),
    '#options' => array(
      'square' => t('Square (50x50)'),
      'small' => t('Small (50 pixels wide, variable height)'),
      'normal' => t('Normal (100 pixels wide, variable height)'),
      'large' => t('Large (about 200 pixels wide, variable height)'),
    ),
    '#default_value' => variable_get('fbapp_user_profile_picture', 'normal'),
    '#description' => t('Image size to retreive from Facebook. The picture will be cached locally and must be within the limits defined in user settings (dimensions and size). Only used if picture support is enabled in user settings.'),
    '#required' => FALSE,
  );

  return system_settings_form($form);
}

