<?php
/**
 * @file
 * Describes Facebook tables and defines specific handlers.
 */

/**
 * Implements hook_views_data().
 */
function fbapp_viewsfql_views_data() {
  $data['fbapp_user'] = array(
    // Table info.
    'table' => array(
      'group' => t('FBApp'),
      'base' => array(
        'field' => 'uid',
        'title' => t('Facebook user'),
        'help' => t('This table contains Facebook\'s users data.'),
        'query class' => 'fbapp_query',
      ),
    ),
    // Fields.
    'uid' => array(
      'title' => t('User UID'),
      'help' => t('The user\'s Facebook UID.'),
      'field' => array(
        'handler' => 'views_handler_field_numeric',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_numeric',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'username' => array(
      'title' => t('Username'),
      'help' => t('The user\'s username.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'email' => array(
      'title' => t('Email'),
      'help' => t('The user\'s email.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'name' => array(
      'title' => t('Full name'),
      'help' => t('The user\'s full name.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'first_name' => array(
      'title' => t('First name'),
      'help' => t('The user\'s first name.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'middle_name' => array(
      'title' => t('Middle name'),
      'help' => t('The user\'s middle name.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'last_name' => array(
      'title' => t('Last name'),
      'help' => t('The user\'s last name.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'user_picture' => array(
      'title' => t('Picture'),
      'help' => t('The user\'s picture.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_user_pic',
      ),
    ),
    'online_presence' => array(
      'title' => t('Online presence'),
      'help' => t('The user\'s online status.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
  );

  $data['fbapp_friend'] = array(
    'table' => array(
      'group' => t('FBApp'),
      'join' => array(
        'fbapp_user' => array(
          'left_field' => 'uid',
          'field' => 'uid1',
        ),
      ),
    ),
    'uid1' => array(
      'title' => t('UID 1'),
      'help' => t('The friend.'),
    ),
    'uid2' => array(
      'title' => t('UID 2'),
      'help' => t('The user.'),
      'argument' => array(
        'handler' => 'fbapp_viewsfql_handler_argument_user_uid',
        'name field' => 'name',
        'numeric' => TRUE,
        'validate type' => 'uid',
      ),
    ),
  );

  $data['fbapp_stream'] = array(
    // Table info.
    'table' => array(
      'group' => t('FBApp'),
      'base' => array(
        'field' => 'post_id',
        'title' => t('Facebook stream'),
        'help' => t('The stream contains posts on Facebook user feeds, pages, applications and groups.'),
        'query class' => 'fbapp_query',
      ),
    ),
    // Post data.
    'post_id' => array(
      'title' => t('Post ID'),
      'help' => t('The Post ID.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'filter_key' => array(
      'title' => t('Filter Key'),
      'help' => t('The Filter Key used to filter the stream table.'),
    ),
    'source_id' => array(
      'title' => t('Source ID'),
      'help' => t('The ID of the user, page, group, or event whose wall the post is on.'),
      'field' => array(
        'handler' => 'views_handler_field_numeric',
        'click sortable' => TRUE,
      ),
      'argument' => array(
        'handler' => 'fbapp_viewsfql_handler_argument_user_uid',
        'name field' => 'name',
        'numeric' => TRUE,
        'validate type' => 'uid',
      ),
      'argument' => array(
        'handler' => 'fbapp_viewsfql_handler_argument_stream_source_id',
        'name field' => 'post_id',
        'numeric' => TRUE,
        'validate type' => 'source_id',
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_numeric',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'actor_id' => array(
      'title' => t('Actor ID'),
      'help' => t('The ID of the user, page, group, or event that published the post.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'message' => array(
      'title' => t('Message'),
      'help' => t('The message written in the post.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'updated_time' => array(
      'title' => t('Last updated time'),
      'help' => t('The time the post was last updated, which occurs when a user comments on the post.'),
      'field' => array(
        'handler' => 'views_handler_field_date',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'views_handler_filter_date',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort_date',
      ),
    ),
    'created_time' => array(
      'title' => t('Created time'),
      'help' => t('The time the post was published.'),
      'field' => array(
        'handler' => 'views_handler_field_date',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'views_handler_filter_date',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort_date',
      ),
    ),
    'attachment' => array(
      'title' => t('Attachment'),
      'help' => t('Data attached to the post.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_attachment',
      ),
    ),
    'likes' => array(
      'title' => t('Likes'),
      'help' => t('List of likes.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_likes',
      ),
    ),
    'comments' => array(
      'title' => t('Comments'),
      'help' => t('List of comments.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_comments',
      ),
    ),
    // Actor info.
    'actor_name' => array(
      'title' => t('Actor\'s full name'),
      'help' => t('The full name of the user, page, group, or event that published the post.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_actor_name',
        'click sortable' => TRUE,
      ),
    ),
    'actor_picture' => array(
      'title' => t('Actor\'s picture'),
      'help' => t('The picture of the user, page, group, or event that published the post.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_actor_pic',
      ),
    ),
  );

  $data['fbapp_stream_filter'] = array(
    // Table info.
    'table' => array(
      'group' => t('FBApp'),
      'join' => array(
        'fbapp_stream' => array(
          'left_field' => 'filter_key',
          'field' => 'filter_key',
        ),
      ),
    ),
    'uid' => array(
      'title' => t('User ID'),
      'help' => t('The ID of the user whose filters you are querying.'),
      'field' => array(
        'handler' => 'views_handler_field_numeric',
        'click sortable' => TRUE,
      ),
      'argument' => array(
        'handler' => 'fbapp_viewsfql_handler_argument_user_uid',
        'name field' => 'name',
        'numeric' => TRUE,
        'validate type' => 'uid',
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_numeric',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'filter_key' => array(
      'title' => t('Filter Key'),
      'help' => t('The Filter Key used to filter the stream table.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
      'sort' => array(
        'handler' => 'views_handler_sort',
      ),
    ),
    'name' => array(
      'title' => t('Filter name'),
      'help' => t('The name of the filter as it appears on the home page.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
    ),
    'type' => array(
      'title' => t('Filter type'),
      'help' => t('The type of filter. One of <em>application</em>, <em>newsfeed</em>, <em>friendlist</em>, <em>network</em>, or <em>public_profiles</em>.'),
      'field' => array(
        'handler' => 'fbapp_viewsfql_handler_field_string',
        'click sortable' => TRUE,
      ),
      'filter' => array(
        'handler' => 'fbapp_viewsfql_handler_filter_string',
      ),
    ),
  );

  return $data;
}

/**
 * Implements hook_views_handlers().
 */
function fbapp_viewsfql_views_handlers() {
  return array(
    'info' => array(
      'path' => drupal_get_path('module', 'fbapp_viewsfql') . '/handlers',
    ),
    'handlers' => array(
      // Arguments.
      'fbapp_viewsfql_handler_argument_user_uid' => array(
        'parent' => 'views_handler_argument',
      ),
      'fbapp_viewsfql_handler_argument_stream_source_id' => array(
        'parent' => 'views_handler_argument',
      ),
      // Fields.
      'fbapp_viewsfql_handler_field_string' => array(
        'parent' => 'views_handler_field',
      ),
      'fbapp_viewsfql_handler_field_user_pic' => array(
        'parent' => 'views_handler_field',
      ),
      'fbapp_viewsfql_handler_field_actor_name' => array(
        'parent' => 'views_handler_field',
      ),
      'fbapp_viewsfql_handler_field_actor_pic' => array(
        'parent' => 'views_handler_field',
      ),
      'fbapp_viewsfql_handler_field_attachment' => array(
        'parent' => 'views_handler_field',
      ),
      'fbapp_viewsfql_handler_field_likes' => array(
        'parent' => 'views_handler_field',
      ),
      'fbapp_viewsfql_handler_field_comments' => array(
        'parent' => 'views_handler_field',
      ),
      // Filters.
      'fbapp_viewsfql_handler_filter_numeric' => array(
        'parent' => 'views_handler_filter_numeric',
      ),
      'fbapp_viewsfql_handler_filter_string' => array(
        'parent' => 'views_handler_filter_string',
      ),
    ),
  );
}

/**
 * Implements hook_views_plugins().
 */
function fbapp_viewsfql_views_plugins() {
  return array(
    'query' => array(
      'fbapp_query' => array(
        'title' => t('Facebook Query Language'),
        'help' => t('Get data through Facebook Graph API using FQL.'),
        'handler' => 'fbapp_viewsfql_plugin_query_graphapi',
      ),
    ),
    'row' => array(
      'fbapp_post' => array(
        'title' => t('Facebook stream'),
        'help' => t('Formats stream views.'),
        'handler' => 'fbapp_viewsfql_plugin_row_post',
        'parent' => 'fields',
        'theme' => 'views_view_feed',
        'uses fields' => TRUE,
        'uses options' => FALSE,
        'type' => 'normal',
      ),
    ),
  );
}

